#!/bin/bash

# Check local-spring-vuln-scanner
# checkmk Local Check
# Christian Wirtz, 2022-04 <doc@snowheaven.de>
#
# https://github.com/hillu/local-spring-vuln-scanner

START_DATE=$(date +%s)

DEST="/" # scan under this path
WAITTIME="10800" # max scan runtime (default: 3h = 10800sec)

EXCLUDE_PATHS="--exclude /mnt --exclude /lost+found --exclude /media --exclude /run/user/"
LOGFILE="/tmp/local-spring-vuln-scanner.log"
ARGS="--verbose --quiet --log $LOGFILE $EXCLUDE_PATHS"
SERVICENAME="CVE-Spring4Shell"
PWD="/usr/lib/check_mk_agent"

# search pattern
PATTERN_VULNERABLE="^indicator for vulnerable component found in "
PATTERN_INSPECTED="^Inspecting "
PATTERN_DENY=": permission denied$"
PATTERN_CVES="^Checking for vulnerabilities: "
PATTERN_REPLACE="indicator for vulnerable component found in "

WAITMAX=$(which waitmax)

COMMAND="$WAITMAX $WAITTIME $PWD/bin/local-spring-vuln-scanner $ARGS $DEST"

# Check if a scan is already running
PROCESSES=$(pgrep -lfc local-spring-vuln-scanner)

# Output if another scan is running
if [[ $PROCESSES -gt 1 ]]
then
	PROCESSES=$(($PROCESSES - 1))
	echo "3 $SERVICENAME processes=$PROCESSES; Another scan is already running, number of processes: $PROCESSES"
	exit 3
fi

eval "$COMMAND"
EXITCODE=$?

CVES=$(grep -E "$PATTERN_CVES" $LOGFILE)
# cut "Checking for vulnerabilities: " from "Checking for vulnerabilities: CVE-2022-22965"
CVES=$(echo "$CVES" | awk -F':' '{print $2}')
# trim spaces at the beginning of string
CVES=${CVES##*( )}

LONGOUTPUT="Checked for: $CVES\nScanned path: $DEST"

DENIED=$(grep -cE "$PATTERN_DENY" $LOGFILE)
VULNERABLE=$(grep -cE "$PATTERN_VULNERABLE" $LOGFILE)
INSPECTED=$(grep -cE "$PATTERN_INSPECTED" $LOGFILE)
UNKNOWN=$(grep -vE "$PATTERN_DENY" $LOGFILE | grep -vE "$PATTERN_VULNERABLE" | grep -vE "$PATTERN_INSPECTED"  | grep -cvE "$PATTERN_CVES")

END_DATE=$(date +%s)
RUN_TIME=$(("$END_DATE" - "$START_DATE"))

# Perfdata
PERFDATA="files_inspected=$INSPECTED|files_vulnerable=$VULNERABLE;1;1|files_not_permitted=$DENIED;1|files_unknown=$UNKNOWN;|run_time=$RUN_TIME;"

# Output if errors while scanning
if [[ $EXITCODE -gt 0 ]]
then
	echo "2 $SERVICENAME - Error on scanner run: $EXITCODE"
	exit 2
fi

# Output if suspicious files found
if [[ $VULNERABLE -eq 0 ]]
then
    MSG="No vulnerabilities found"
else
  # get files with indicator
  FILES_VULNERABLE=$(sed -n -e "/$PATTERN_VULNERABLE/p" $LOGFILE | sed "s/$PATTERN_REPLACE//g" | sed ':a;N;$!ba;s/\n/\\n/g')
  FILES_VULNERABLE="\nIndicator for vulnerable component found in:\n$FILES_VULNERABLE"
  MSG="Found indicators for vulnerable components"
  LONGOUTPUT="$LONGOUTPUT\n$FILES_VULNERABLE"
fi

if [[ DENIED -gt 0 ]]
then
  # get denied files
  FILES_DENIED=$(sed -n -e "/$PATTERN_DENY/p" $LOGFILE | sed ':a;N;$!ba;s/\n/\\n/g')
  FILES_DENIED="\nUnscanned files:\n$FILES_DENIED"
  LONGOUTPUT="$LONGOUTPUT\n$FILES_DENIED"
fi

if [[ UNKNOWN -gt 0 ]]
then
  # get unknown files
  FILES_UNKNOWN=$(grep -vE "$PATTERN_DENY" $LOGFILE | grep -vE "$PATTERN_VULNERABLE" | grep -vE "$PATTERN_INSPECTED"  | grep -cvE "$PATTERN_CVES")
  FILES_UNKNOWN=$(echo "$FILES_UNKNOWN" | sed ':a;N;$!ba;s/\n/\\n/g')
  FILES_UNKNOWN="\nFiles with unknown state:\n$FILES_UNKNOWN"
  LONGOUTPUT="$LONGOUTPUT\n$FILES_UNKNOWN"
fi

# Default output
echo "P $SERVICENAME $PERFDATA $MSG\n$LONGOUTPUT\n"

# cleanup
unset ARGS
unset COMMAND
unset CVES
unset DENIED
unset DEST
unset END_DATE
unset EXITCODE
unset FILES_DENIED
unset FILES_UNKNOWN
unset FILES_VULNERABLE
unset INSPECTED
unset LOGFILE
unset LONGOUTPUT
unset MSG
unset PATTERN_CVES
unset PATTERN_DENY
unset PATTERN_INSPECTED
unset PATTERN_REPLACE
unset PATTERN_VULNERABLE
unset PERFDATA
unset PROCESSES
unset PWD
unset RUN_TIME
unset SERVICENAME
unset START_DATE
unset UNKNOWN
unset VULNERABLE
unset WAITMAX
unset WAITTIME

exit 0
