<#
.Synopsis
   Checks for CVE-2022-22965 wrapper for checkmk for a scan tool to check all drives and parse the output
.DESCRIPTION
   Author: Christopher Stegmann
   Date: 2022-04
#>

$START_TIME = Get-Date

$ErrorActionPreference = "SilentlyContinue"
$SVC_NAME = "CVE-Spring4Shell"

# pattern to search in output
$PATTERN_VULNERABLE="^indicator for vulnerable component found in "
$PATTERN_INSPECTING="^Inspecting "
$PATTERN_DENY=": Access is denied\.$"
$PATTERN_CVES="^Checking for vulnerabilities: "
$PATTERN_REPLACE="indicator for vulnerable component found in "

# get list of drives to check without empty drives (like CDROM), returns i.e. "C:\ D:\"
$DRIVES_TO_CHECK = (Get-PSDrive -PSProvider "FileSystem" | Where-Object used -gt 0 | Select -ExpandProperty root) -join " "

$LONGOUTPUT = "Scanned drive(s): $($DRIVES_TO_CHECK)"

$EXECUTABLE = "c:\ProgramData\checkmk\agent\bin\local-spring-vuln-scanner.exe"
$EXCLUDE_PATHS = "--exclude ""C:\System Volume Information"""
$LOGFILE = "$env:TEMP\spring4shell.log"
$ARGS = "--verbose --quiet --log $($LOGFILE) $($EXCLUDE_PATHS)"

$RUN = "$EXECUTABLE $ARGS $DRIVES_TO_CHECK"

if (Test-Path -Path $LOGFILE -PathType Leaf) {
    Remove-Item -Path $LOGFILE
}

if ( -not (Test-Path -Path $EXECUTABLE -PathType Leaf)) {
  # warn scanner not found
  Write-Output "1 $($SVC_NAME) - Error: $($EXECUTABLE) not found"
  return
}

# run the scanner tool
try{
    & cmd /c $RUN
}
catch {
    $ERRORINFO = $_
    # warn error on scanning
    Write-output "1 $($SVC_NAME) - Error on scanner run: $($ERRORINFO)"
    return
}

if (Test-Path -Path "$LOGFILE" -PathType Leaf) {
  # search for CVEs checked for:
  $CVES = (Select-String -Path $LOGFILE -CaseSensitive -Pattern $PATTERN_CVES | Select -ExpandProperty Line).split(":")[1].trim()
  $LONGOUTPUT="$LONGOUTPUT\nChecked for: $CVES"

  # search for vulnerabilities and remove log file name from output:
  $FILES_VULNERABLE = Select-String -Path $LOGFILE -CaseSensitive -Pattern $PATTERN_VULNERABLE | Select -ExpandProperty Line
  # $FILES_VULNERABLE.Length does not work for one line match, it will then count the chars not the lines
  $VULNERABLE = $FILES_VULNERABLE | Measure-Object | Select-Object -ExpandProperty count
  if ( $VULNERABLE -gt 0 ) {
    $FILES_VULNERABLE=($FILES_VULNERABLE -join "\n") -replace $PATTERN_REPLACE, ""
    $FILES_VULNERABLE = "\nIndicator for vulnerable component found in:\n$FILES_VULNERABLE"
    $MSG = "Found indicators for vulnerable components"
    $LONGOUTPUT="$LONGOUTPUT\n$FILES_VULNERABLE"
  } else {
    # $MSG = "$MSG, No vulnerabilities found"
    $MSG = "No vulnerabilities found"
  }

  # search for denied files and remove log file name from output:
  $FILES_DENIED= (Select-String -Path $LOGFILE -CaseSensitive -Pattern $PATTERN_DENY) | Select -ExpandProperty Line
  $DENIED = $FILES_DENIED | Measure-Object | Select-Object -ExpandProperty count
  if ( $DENIED -gt 0 ) {
    $FILES_DENIED=($FILES_DENIED -join "\n")
    $FILES_DENIED = "\nUnscanned files:\n$FILES_DENIED"
    $LONGOUTPUT="$LONGOUTPUT\n$FILES_DENIED"
  }

  # get anything else
  $FILES_UNKNOWN = Select-String -Path $LOGFILE -CaseSensitive -Pattern $PATTERN_VULNERABLE -NotMatch | Select -ExpandProperty Line | Select-String -Pattern $PATTERN_DENY -NotMatch | Select-String -Pattern $PATTERN_INSPECTING -NotMatch | Select-String -Pattern $PATTERN_CVES -NotMatch
  $UNKNOWN = $FILES_UNKNOWN | Measure-Object | Select-Object -ExpandProperty count
  if ( $UNKNOWN -gt 0 ) {
    $FILES_UNKNOWN=($FILES_UNKNOWN -join "\n")
    $FILES_UNKNOWN = "\nFiles with unknown state:\n$FILES_UNKNOWN"
    $LONGOUTPUT="$LONGOUTPUT\n$FILES_UNKNOWN"
  }

  # get number of inspected .jar/.war files
  $FILES_INSPECTED = Select-String -Path $LOGFILE -CaseSensitive -Pattern $PATTERN_INSPECTING | Select -ExpandProperty Line
  $INSPECTED = $FILES_INSPECTED | Measure-Object | Select-Object -ExpandProperty count

  # remove log file
  Remove-Item -Path $LOGFILE

} else {
   Write-output "1 $($SVC_NAME) Logfile $($LOGFILE) not found"
   return
}

$END_TIME = Get-Date
$RUN_TIME = (New-TimeSpan -Start $START_TIME -End $END_TIME).TotalSeconds
$PERFDATA="files_inspected=$INSPECTED|files_vulnerable=$VULNERABLE;1;1|files_not_permitted=$DENIED;1|files_unknown=$UNKNOWN;|run_time=$RUN_TIME;"
write-output "P $($SVC_NAME) $PERFDATA $MSG\n$LONGOUTPUT\n"

exit 0
